﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

using UnityEngine;
using HydroTech_FC;

[KSPAddon(KSPAddon.Startup.Flight, false)]
public partial class HydroMouseControlModule : MonoBehaviour
{
    static private readonly FileName.Folder saveFolder = new FileName.Folder(FileName.HydroTechFolder, "PluginData", "mousecontrol");
    HydroImageLoader picture = new HydroImageLoader(
        new FileName("Panel200", "png", saveFolder),
        215,
        215
        );

    public void Awake()
    {
        picture.Load();
        settings.Load();
        xValue.Set(picture.Width, 5, 5);
        yValue.Set(picture.Height, 5, 5);
    }

    Vector2 pictureOffset = new Vector2(20, 82);
    Rect pictureRect
    {
        get
        {
            return new Rect(
                windowRect.xMin + pictureOffset.x,
                windowRect.yMin + pictureOffset.y,
                picture.Width,
                picture.Height
                );
        }
    }
    Vector2 MousePosition
    {
        get
        {
            return new Vector2(
                Input.mousePosition.x,
                Screen.height - Input.mousePosition.y
                );
        }
    }

    void Drive(FlightCtrlState ctrlState)
    {
        if (!Engaged)
            return;
        if (curVessel != FlightGlobals.ActiveVessel)
            SwitchVessel(FlightGlobals.ActiveVessel);
        xValue.pixels = (int)(MousePosition.x - pictureRect.center.x);
        yValue.pixels = (int)(pictureRect.center.y - MousePosition.y);
        if (xControl.CheckOutArea(xValue) || yControl.CheckOutArea(yValue))
            state = STATES.OUT;
        else if (xControl.CheckSuspend(FlightInputHandler.state) || yControl.CheckSuspend(FlightInputHandler.state))
            state = STATES.SUSPEND;
        else
        {
            state = STATES.OK;
            xControl.SetFlightCtrl(FlightGlobals.ActiveVessel.ctrlState, xValue);
            yControl.SetFlightCtrl(FlightGlobals.ActiveVessel.ctrlState, yValue);
        }
    }

    Vessel curVessel = null;

    void SwitchVessel(Vessel newctl)
    {
        if (curVessel)
            curVessel.OnFlyByWire -= Drive;
        curVessel = newctl;
        if (curVessel)
           curVessel.OnFlyByWire += Drive;
    }

    bool _Engaged = false;
    bool Engaged
    {
        get { return _Engaged; }
        set
        {
            if (value != _Engaged)
            {
                if (value)
                {
                    float width = windowRect.width;
                    float height = windowRect.height;
                    settings.windowRect.xMin = MousePosition.x - picture.Width * 0.5F - pictureOffset.x;
                    settings.windowRect.yMin = MousePosition.y - picture.Height * 0.5F - pictureOffset.y;
                    settings.windowRect.width = width;
                    settings.windowRect.height = height;
                    SwitchVessel(FlightGlobals.ActiveVessel);
                }
                else
                    SwitchVessel(null);
            }
            _Engaged = value;
        }
    }

    SETTINGS settings = new SETTINGS();
    CONTROL xControl
    {
        get { return settings.xControl; }
        set { settings.xControl = value; }
    }
    VALUE xValue = new VALUE();
    CONTROL yControl
    {
        get { return settings.yControl; }
        set { settings.yControl = value; }
    }
    VALUE yValue = new VALUE();
    bool label
    {
        get { return settings.label; }
        set { settings.label = value; }
    }
    enum STATES { OK, OUT, SUSPEND }
    STATES state = STATES.OK;
}